@@ -30,8 +30,6 @@ use std::ops::IndexMut;
 use std::ops::RangeInclusive;
 use std::time;
 
-use ring::aead;
-
 use crate::Error;
 use crate::Result;
 
@@ -53,6 +51,8 @@ pub const MAX_PKT_NUM_LEN: usize = 4;
 
 const SAMPLE_LEN: usize = 16;
 
+const RETRY_AEAD_ALG: crypto::Algorithm = crypto::Algorithm::AES128_GCM;
+
 #[derive(Clone, Copy, PartialEq, Eq, PartialOrd, Ord, Debug)]
 pub enum Epoch {
     Initial     = 0,
@@ -201,23 +201,23 @@ impl<'a> ConnectionId<'a> {
     }
 }
 
-impl<'a> Default for ConnectionId<'a> {
+impl Default for ConnectionId<'_> {
     #[inline]
     fn default() -> Self {
         Self::from_vec(Vec::new())
     }
 }
 
-impl<'a> From<Vec<u8>> for ConnectionId<'a> {
+impl From<Vec<u8>> for ConnectionId<'_> {
     #[inline]
     fn from(v: Vec<u8>) -> Self {
         Self::from_vec(v)
     }
 }
 
-impl<'a> From<ConnectionId<'a>> for Vec<u8> {
+impl From<ConnectionId<'_>> for Vec<u8> {
     #[inline]
-    fn from(id: ConnectionId<'a>) -> Self {
+    fn from(id: ConnectionId<'_>) -> Self {
         match id.0 {
             ConnectionIdInner::Vec(cid) => cid,
             ConnectionIdInner::Ref(cid) => cid.to_vec(),
@@ -225,16 +225,16 @@ impl<'a> From<ConnectionId<'a>> for Vec<u8> {
     }
 }
 
-impl<'a> PartialEq for ConnectionId<'a> {
+impl PartialEq for ConnectionId<'_> {
     #[inline]
     fn eq(&self, other: &Self) -> bool {
         self.as_ref() == other.as_ref()
     }
 }
 
-impl<'a> Eq for ConnectionId<'a> {}
+impl Eq for ConnectionId<'_> {}
 
-impl<'a> AsRef<[u8]> for ConnectionId<'a> {
+impl AsRef<[u8]> for ConnectionId<'_> {
     #[inline]
     fn as_ref(&self) -> &[u8] {
         match &self.0 {
@@ -244,14 +244,14 @@ impl<'a> AsRef<[u8]> for ConnectionId<'a> {
     }
 }
 
-impl<'a> std::hash::Hash for ConnectionId<'a> {
+impl std::hash::Hash for ConnectionId<'_> {
     #[inline]
     fn hash<H: std::hash::Hasher>(&self, state: &mut H) {
         self.as_ref().hash(state);
     }
 }
 
-impl<'a> std::ops::Deref for ConnectionId<'a> {
+impl std::ops::Deref for ConnectionId<'_> {
     type Target = [u8];
 
     #[inline]
@@ -263,14 +263,14 @@ impl<'a> std::ops::Deref for ConnectionId<'a> {
     }
 }
 
-impl<'a> Clone for ConnectionId<'a> {
+impl Clone for ConnectionId<'_> {
     #[inline]
     fn clone(&self) -> Self {
         Self::from_vec(self.as_ref().to_vec())
     }
 }
 
-impl<'a> std::fmt::Debug for ConnectionId<'a> {
+impl std::fmt::Debug for ConnectionId<'_> {
     #[inline]
     fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
         for c in self.as_ref() {
@@ -403,12 +403,14 @@ impl<'a> Header<'a> {
             },
 
             Type::Retry => {
+                const TAG_LEN: usize = RETRY_AEAD_ALG.tag_len();
+
                 // Exclude the integrity tag from the token.
-                if b.cap() < aead::AES_128_GCM.tag_len() {
+                if b.cap() < TAG_LEN {
                     return Err(Error::InvalidPacket);
                 }
 
-                let token_len = b.cap() - aead::AES_128_GCM.tag_len();
+                let token_len = b.cap() - TAG_LEN;
                 token = Some(b.get_bytes(token_len)?.to_vec());
             },
 
@@ -522,7 +524,7 @@ impl<'a> Header<'a> {
     }
 }
 
-impl<'a> std::fmt::Debug for Header<'a> {
+impl std::fmt::Debug for Header<'_> {
     fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
         write!(f, "{:?}", self.ty)?;
 
@@ -776,26 +778,25 @@ pub fn retry(
 pub fn verify_retry_integrity(
     b: &octets::OctetsMut, odcid: &[u8], version: u32,
 ) -> Result<()> {
+    const TAG_LEN: usize = RETRY_AEAD_ALG.tag_len();
+
     let tag = compute_retry_integrity_tag(b, odcid, version)?;
 
-    ring::constant_time::verify_slices_are_equal(
-        &b.as_ref()[..aead::AES_128_GCM.tag_len()],
-        tag.as_ref(),
-    )
-    .map_err(|_| Error::CryptoFail)?;
-
-    Ok(())
+    crypto::verify_slices_are_equal(&b.as_ref()[..TAG_LEN], tag.as_ref())
 }
 
 fn compute_retry_integrity_tag(
     b: &octets::OctetsMut, odcid: &[u8], version: u32,
-) -> Result<aead::Tag> {
-    const RETRY_INTEGRITY_KEY_V1: [u8; 16] = [
+) -> Result<Vec<u8>> {
+    const KEY_LEN: usize = RETRY_AEAD_ALG.key_len();
+    const TAG_LEN: usize = RETRY_AEAD_ALG.tag_len();
+
+    const RETRY_INTEGRITY_KEY_V1: [u8; KEY_LEN] = [
         0xbe, 0x0c, 0x69, 0x0b, 0x9f, 0x66, 0x57, 0x5a, 0x1d, 0x76, 0x6b, 0x54,
         0xe3, 0x68, 0xc8, 0x4e,
     ];
 
-    const RETRY_INTEGRITY_NONCE_V1: [u8; aead::NONCE_LEN] = [
+    const RETRY_INTEGRITY_NONCE_V1: [u8; crypto::MAX_NONCE_LEN] = [
         0x46, 0x15, 0x99, 0xd3, 0x5d, 0x63, 0x2b, 0xf2, 0x23, 0x98, 0x25, 0xbb,
     ];
 
@@ -816,17 +817,23 @@ fn compute_retry_integrity_tag(
     pb.put_bytes(odcid)?;
     pb.put_bytes(&b.buf()[..hdr_len])?;
 
-    let key = aead::LessSafeKey::new(
-        aead::UnboundKey::new(&aead::AES_128_GCM, key)
-            .map_err(|_| Error::CryptoFail)?,
-    );
+    let key = crypto::PacketKey::new(
+        RETRY_AEAD_ALG,
+        key.to_vec(),
+        nonce.to_vec(),
+        crypto::Seal::ENCRYPT,
+    )?;
 
-    let nonce = aead::Nonce::assume_unique_for_key(nonce);
+    let mut out_tag = vec![0_u8; TAG_LEN];
 
-    let aad = aead::Aad::from(&pseudo);
+    let out_len = key.seal_with_u64_counter(0, &pseudo, &mut out_tag, 0, None)?;
 
-    key.seal_in_place_separate_tag(nonce, aad, &mut [])
-        .map_err(|_| Error::CryptoFail)
+    // Ensure that the output only contains the AEAD tag.
+    if out_len != out_tag.len() {
+        return Err(Error::CryptoFail);
+    }
+
+    Ok(out_tag)
 }
 
 pub struct KeyUpdate {
@@ -852,8 +859,6 @@ pub struct PktNumSpace {
 
     pub largest_rx_non_probing_pkt_num: u64,
 
-    pub next_pkt_num: u64,
-
     pub recv_pkt_need_ack: ranges::RangeSet,
 
     pub recv_pkt_num: PktNumWindow,
@@ -879,8 +884,6 @@ impl PktNumSpace {
 
             largest_rx_non_probing_pkt_num: 0,
 
-            next_pkt_num: 0,
-
             recv_pkt_need_ack: ranges::RangeSet::new(crate::MAX_ACK_RANGES),
 
             recv_pkt_num: PktNumWindow::default(),
@@ -1524,7 +1527,7 @@ mod tests {
 
         let alg = crypto::Algorithm::ChaCha20_Poly1305;
 
-        let aead = crypto::Open::from_secret(alg, secret.into()).unwrap();
+        let aead = crypto::Open::from_secret(alg, &secret).unwrap();
 
         let mut hdr = Header::from_bytes(&mut b, 0).unwrap();
         assert_eq!(hdr.ty, Type::Short);
@@ -1892,7 +1895,7 @@ mod tests {
 
         let alg = crypto::Algorithm::ChaCha20_Poly1305;
 
-        let aead = crypto::Seal::from_secret(alg, secret.into()).unwrap();
+        let aead = crypto::Seal::from_secret(alg, &secret).unwrap();
 
         let pn = 654_360_564;
         let pn_len = 3;
