@@ -30,7 +30,7 @@ use std::slice;
 
 use std::io::Write;
 
-use once_cell::sync::Lazy;
+use std::sync::LazyLock;
 
 use libc::c_char;
 use libc::c_int;
@@ -123,9 +123,7 @@ enum ssl_private_key_result_t {
 }
 
 /// BoringSSL ex_data index for quiche connections.
-///
-/// TODO: replace with `std::sync::LazyLock` when stable.
-pub static QUICHE_EX_DATA_INDEX: Lazy<c_int> = Lazy::new(|| unsafe {
+pub static QUICHE_EX_DATA_INDEX: LazyLock<c_int> = LazyLock::new(|| unsafe {
     SSL_get_ex_new_index(0, ptr::null(), ptr::null(), ptr::null(), ptr::null())
 });
 
@@ -214,10 +212,11 @@ impl Context {
     fn load_ca_certs(&mut self) -> Result<()> {
         unsafe {
             let cstr = ffi::CString::new("Root").map_err(|_| Error::TlsFail)?;
-            let sys_store = winapi::um::wincrypt::CertOpenSystemStoreA(
-                0,
-                cstr.as_ptr() as winapi::um::winnt::LPCSTR,
-            );
+            let sys_store =
+                windows_sys::Win32::Security::Cryptography::CertOpenSystemStoreA(
+                    0,
+                    cstr.as_ptr() as windows_sys::core::PCSTR,
+                );
             if sys_store.is_null() {
                 return Err(Error::TlsFail);
             }
@@ -227,7 +226,7 @@ impl Context {
                 return Err(Error::TlsFail);
             }
 
-            let mut ctx_p = winapi::um::wincrypt::CertEnumCertificatesInStore(
+            let mut ctx_p = windows_sys::Win32::Security::Cryptography::CertEnumCertificatesInStore(
                 sys_store,
                 ptr::null(),
             );
@@ -246,14 +245,16 @@ impl Context {
 
                 X509_free(cert);
 
-                ctx_p = winapi::um::wincrypt::CertEnumCertificatesInStore(
+                ctx_p = windows_sys::Win32::Security::Cryptography::CertEnumCertificatesInStore(
                     sys_store, ctx_p,
                 );
             }
 
             // tidy up
-            winapi::um::wincrypt::CertFreeCertificateContext(ctx_p);
-            winapi::um::wincrypt::CertCloseStore(sys_store, 0);
+            windows_sys::Win32::Security::Cryptography::CertFreeCertificateContext(ctx_p);
+            windows_sys::Win32::Security::Cryptography::CertCloseStore(
+                sys_store, 0,
+            );
         }
 
         Ok(())
@@ -715,7 +716,7 @@ fn get_cipher_from_ptr(cipher: *const SSL_CIPHER) -> Result<crypto::Algorithm> {
     Ok(alg)
 }
 
-extern fn set_read_secret(
+extern "C" fn set_read_secret(
     ssl: *mut SSL, level: crypto::Level, cipher: *const SSL_CIPHER,
     secret: *const u8, secret_len: usize,
 ) -> c_int {
@@ -749,7 +750,7 @@ extern fn set_read_secret(
     if level != crypto::Level::ZeroRTT || ex_data.is_server {
         let secret = unsafe { slice::from_raw_parts(secret, secret_len) };
 
-        let open = match crypto::Open::from_secret(aead, secret.to_vec()) {
+        let open = match crypto::Open::from_secret(aead, secret) {
             Ok(v) => v,
 
             Err(_) => return 0,
@@ -766,7 +767,7 @@ extern fn set_read_secret(
     1
 }
 
-extern fn set_write_secret(
+extern "C" fn set_write_secret(
     ssl: *mut SSL, level: crypto::Level, cipher: *const SSL_CIPHER,
     secret: *const u8, secret_len: usize,
 ) -> c_int {
@@ -800,7 +801,7 @@ extern fn set_write_secret(
     if level != crypto::Level::ZeroRTT || !ex_data.is_server {
         let secret = unsafe { slice::from_raw_parts(secret, secret_len) };
 
-        let seal = match crypto::Seal::from_secret(aead, secret.to_vec()) {
+        let seal = match crypto::Seal::from_secret(aead, secret) {
             Ok(v) => v,
 
             Err(_) => return 0,
@@ -812,7 +813,7 @@ extern fn set_write_secret(
     1
 }
 
-extern fn add_handshake_data(
+extern "C" fn add_handshake_data(
     ssl: *mut SSL, level: crypto::Level, data: *const u8, len: usize,
 ) -> c_int {
     let ex_data = match get_ex_data_from_ptr::<ExData>(ssl, *QUICHE_EX_DATA_INDEX)
@@ -848,14 +849,16 @@ extern fn add_handshake_data(
     1
 }
 
-extern fn flush_flight(_ssl: *mut SSL) -> c_int {
+extern "C" fn flush_flight(_ssl: *mut SSL) -> c_int {
     // We don't really need to anything here since the output packets are
     // generated separately, when conn.send() is called.
 
     1
 }
 
-extern fn send_alert(ssl: *mut SSL, level: crypto::Level, alert: u8) -> c_int {
+extern "C" fn send_alert(
+    ssl: *mut SSL, level: crypto::Level, alert: u8,
+) -> c_int {
     let ex_data = match get_ex_data_from_ptr::<ExData>(ssl, *QUICHE_EX_DATA_INDEX)
     {
         Some(v) => v,
@@ -880,7 +883,7 @@ extern fn send_alert(ssl: *mut SSL, level: crypto::Level, alert: u8) -> c_int {
     1
 }
 
-extern fn keylog(ssl: *const SSL, line: *const c_char) {
+extern "C" fn keylog(ssl: *const SSL, line: *const c_char) {
     let ex_data = match get_ex_data_from_ptr::<ExData>(ssl, *QUICHE_EX_DATA_INDEX)
     {
         Some(v) => v,
@@ -900,7 +903,7 @@ extern fn keylog(ssl: *const SSL, line: *const c_char) {
     }
 }
 
-extern fn select_alpn(
+extern "C" fn select_alpn(
     ssl: *mut SSL, out: *mut *const u8, out_len: *mut u8, inp: *mut u8,
     in_len: c_uint, _arg: *mut c_void,
 ) -> c_int {
@@ -960,7 +963,7 @@ extern fn select_alpn(
     TLS_ERROR
 }
 
-extern fn new_session(ssl: *mut SSL, session: *mut SSL_SESSION) -> c_int {
+extern "C" fn new_session(ssl: *mut SSL, session: *mut SSL_SESSION) -> c_int {
     let ex_data = match get_ex_data_from_ptr::<ExData>(ssl, *QUICHE_EX_DATA_INDEX)
     {
         Some(v) => v,
@@ -1044,7 +1047,7 @@ fn log_ssl_error() {
     trace!("{}", std::str::from_utf8(&err).unwrap());
 }
 
-extern {
+extern "C" {
     // Note: some vendor-specific methods are implemented by each vendor's
     // submodule (openssl-quictls / boringssl).
 
@@ -1076,13 +1079,16 @@ extern {
     fn SSL_CTX_set_verify(
         ctx: *mut SSL_CTX, mode: c_int,
         cb: Option<
-            unsafe extern fn(ok: c_int, store_ctx: *mut X509_STORE_CTX) -> c_int,
+            unsafe extern "C" fn(
+                ok: c_int,
+                store_ctx: *mut X509_STORE_CTX,
+            ) -> c_int,
         >,
     );
 
     fn SSL_CTX_set_keylog_callback(
         ctx: *mut SSL_CTX,
-        cb: Option<unsafe extern fn(ssl: *const SSL, line: *const c_char)>,
+        cb: Option<unsafe extern "C" fn(ssl: *const SSL, line: *const c_char)>,
     );
 
     fn SSL_CTX_set_alpn_protos(
@@ -1092,7 +1098,7 @@ extern {
     fn SSL_CTX_set_alpn_select_cb(
         ctx: *mut SSL_CTX,
         cb: Option<
-            unsafe extern fn(
+            unsafe extern "C" fn(
                 ssl: *mut SSL,
                 out: *mut *const u8,
                 out_len: *mut u8,
@@ -1107,7 +1113,10 @@ extern {
     fn SSL_CTX_sess_set_new_cb(
         ctx: *mut SSL_CTX,
         cb: Option<
-            unsafe extern fn(ssl: *mut SSL, session: *mut SSL_SESSION) -> c_int,
+            unsafe extern "C" fn(
+                ssl: *mut SSL,
+                session: *mut SSL_SESSION,
+            ) -> c_int,
         >,
     );
 
